/*
 * Node-tar
 * MIT License (c) 2011 Tim Caswell, T. Jameson Little
 */

(function () {
	"use strict";

	var fs = require('fs'),
		path = require('path'),
		Stream = require('stream').Stream,
		header = require("./header"),
		utils = require("./utils"),
		forEachAsync = require('futures').forEachAsync;
	
	function Tar(opt) {
		var tape;

		Stream.apply(this, arguments);

		// I know, it's kinda ugly, but I really don't want an if-else...
		this.addHidden = (opt && opt.addHidden !== undefined) ? opt.addHidden : false;
		this.consolidate = (opt && opt.consolidate !== undefined) ? opt.consolidate : false;
		this.normalize = (opt && opt.normalize !== undefined) ? opt.normalize : true;
		this.recurse = (opt && opt.recurse !== undefined) ? opt.recurse : true;

		this.written = 0;

		tape = this;

		this.addListener('data', function (chunk) {
			tape.written += chunk.length;
		});
		this.addListener('end', function () {
			this.emit('data', utils.clean(10240 - (this.written % 10240)));
		});

		// this seems to need to be done after the listeners are added
		if (opt && opt.output) {
			this.pipe(opt.output);
		}
	}

	Tar.prototype = Object.create(Stream.prototype, {
		constructor: { value: Tar }
	});

	Tar.prototype.close = function () {
		this.emit('end');
	};

	Tar.prototype.append = function (filepath, callback) {
		var tape = this;

		fs.lstat(filepath, function (err, stat) {
			var type, data = {}, mtime, filesize;

			function finish(cb) {
				var checksum = 0, written, input;
				Object.keys(data).forEach(function (key) {
					var i, value = data[key];

					for (i = 0; i < value.length; i += 1) {
						checksum += value.charCodeAt(i);
					}
				});

				data.checksum = utils.pad(checksum, 6) + "\u0000 ";
				tape.emit('data', header.format(data));

				if (stat.isFile()) {
					written = 0;
					input = fs.createReadStream(filepath);
					input.on('data', function (chunk) {
						written += chunk.length;
						tape.emit('data', chunk);
					});
					input.on('error', function (err) {
						tape.emit('error', err);
						cb(err);
					});
					input.on('end', function () {
						tape.emit('data', utils.clean(512 - (written % 512)));
						cb();
					});
				} else {
					cb();
				}
			}

			if (err) {
				callback(err);
				return tape.emit('error', err);
			}

			try {
				if (stat.isDirectory()) {
					filepath += "/";
					type = "5";
				} else if (stat.isSymbolicLink()) {
					type = "2";
				} else if (stat.isFile()) {
					type = "0";
				} else {
					type = new Error();
				}

				mtime = stat.mtime.getTime() / 1000;
				filesize = stat.isFile() ? stat.size : 0;

				data = {
					fileName: tape.consolidate ? path.basename(filepath) : filepath,
					fileMode: utils.pad((stat.mode & 0xfff), 7),
					uid: utils.pad(stat.uid, 7),
					gid: utils.pad(stat.gid, 7),
					fileSize: utils.pad(filesize, 11),
					mtime: utils.pad(mtime, 11),
					checksum: "        ",
					type: type,
					ustar: "ustar  ",
					owner: "",
					group: ""
				};
		
				if (tape.normalize && !tape.consolidate) {
					data.fileName = path.normalize(data.fileName);
				}

				if (stat.isSymbolicLink()) {
					fs.readlink(filepath, function (err, resolvedPath) {
						if (err) {
							callback(err);
							return tape.emit('error', err);
						}
						data.linkName = resolvedPath;	
						finish(callback);
					});
				} else {
					finish(callback);
				}
			} catch (err2) {
				tape.emit('error', err2);
				callback(err2);
			}
		});
	};

	Tar.prototype.addFiles = function (files, callback) {
		var error, tape = this;

		forEachAsync(files, function (next, filename) {
			// make sure we get through all cases
			if (error) {
				next();
			}

			fs.lstat(filename, function (err, stat) {
				if (err) {
					error = err;
					return next();
				}

				if (!tape.addHidden && path.basename(filename).charAt(0) === '.') {
					return next();
				}

				// we won't even append the folders if we don't want to recurse
				if (!tape.recurse && stat.isDirectory()) {
					return next();
				}

				tape.append(filename, function (tErr) {
					if (tErr) {
						error = tErr;
						return next();
					}

					if (stat.isDirectory()) {
						tape.addDirectory(filename, function (tErr2) {
							if (tErr2) {
								error = tErr2;
								return next();
							}

							next();
						});
					} else {
						next();
					}
				});
			});
		}).then(function () {
			callback(error);
		});
	};

	Tar.prototype.addDirectory = function (filepath, callback) {
		var tape = this;
		fs.readdir(filepath, function (err, filenames) {
			if (err) {
				return callback(err);
			}

			filenames.forEach(function (filename, i, arr) {
				if (tape.normalize) {
					arr[i] = path.join(filepath, filename);
				} else {
					arr[i] = filepath + "/" + filename;
				}
			});

			return tape.addFiles.apply(tape, [filenames, callback]);
		});
	};
	
	module.exports = Tar;
}());
